@@ -2,8 +2,8 @@
 from ctypes import Structure, \
                    CFUNCTYPE, POINTER, addressof, sizeof, cast, \
                    c_short, c_int, c_uint, c_size_t, c_long, \
-                   c_uint8, c_uint16, \
-                   c_void_p, c_char, c_char_p, py_object, string_at
+                   c_uint8, c_uint16, c_uint32, \
+                   c_void_p, c_char_p, py_object, string_at
 from ctypes.util import find_library
 import platform
 import os.path
@@ -104,7 +104,9 @@ libusb_class_code = Enum({
 'LIBUSB_CLASS_COMM': 2,
 # Human Interface Device class
 'LIBUSB_CLASS_HID': 3,
-# Printer dclass
+# Physical
+'LIBUSB_CLASS_PHYSICAL': 5,
+# Printer class
 'LIBUSB_CLASS_PRINTER': 7,
 # Picture transfer protocol class
 'LIBUSB_CLASS_PTP': 6,
@@ -114,6 +116,16 @@ libusb_class_code = Enum({
 'LIBUSB_CLASS_HUB': 9,
 # Data class
 'LIBUSB_CLASS_DATA': 10,
+# Smart Card
+'LIBUSB_CLASS_SMART_CARD': 0x0b,
+# Content Security
+'LIBUSB_CLASS_CONTENT_SECURITY': 0x0d,
+# Video
+'LIBUSB_CLASS_VIDEO': 0x0e,
+# Personal Healthcare
+'LIBUSB_CLASS_PERSONAL_HEALTHCARE': 0x0f,
+# Diagnostic Device
+'LIBUSB_CLASS_DIAGNOSTIC_DEVICE': 0xdc,
 # Wireless class
 'LIBUSB_CLASS_WIRELESS': 0xe0,
 # Application class
@@ -121,6 +133,7 @@ libusb_class_code = Enum({
 # Class is vendor-specific
 'LIBUSB_CLASS_VENDOR_SPEC': 0xff
 })
+LIBUSB_CLASS_IMAGE = LIBUSB_CLASS_PTP
 
 # Descriptor types as defined by the USB specification.
 libusb_descriptor_type = Enum({
@@ -346,7 +359,7 @@ libusb_config_descriptor_p = POINTER(libusb_config_descriptor)
 libusb_config_descriptor_p_p = POINTER(libusb_config_descriptor_p)
 
 class libusb_control_setup(Structure):
-    _fields_ = [('bRequestType', c_uint8),
+    _fields_ = [('bmRequestType', c_uint8),
                 ('bRequest', c_uint8),
                 ('wValue', c_uint16),
                 ('wIndex', c_uint16),
@@ -405,6 +418,19 @@ class libusb_device_handle(Structure):
 libusb_device_handle_p = POINTER(libusb_device_handle)
 libusb_device_handle_p_p = POINTER(libusb_device_handle_p)
 
+libusb_speed = Enum({
+# The OS doesn't report or know the device speed.
+'LIBUSB_SPEED_UNKNOWN': 0,
+# The device is operating at low speed (1.5MBit/s).
+'LIBUSB_SPEED_LOW': 1,
+# The device is operating at full speed (12MBit/s).
+'LIBUSB_SPEED_FULL': 2,
+# The device is operating at high speed (480MBit/s).
+'LIBUSB_SPEED_HIGH': 3,
+# The device is operating at super speed (5000MBit/s).
+'LIBUSB_SPEED_SUPER': 4,
+})
+
 # Error codes. Most libusb functions return 0 on success or one of these
 # codes on failure.
 libusb_error = Enum({
@@ -499,6 +525,11 @@ libusb_transfer._fields_ = [('dev_handle', libusb_device_handle_p),
                             ('iso_packet_desc', libusb_iso_packet_descriptor)
 ]
 
+libusb_capability = Enum({
+# The libusb_has_capability() API is available.
+'LIBUSB_CAP_HAS_CAPABILITY': 0,
+})
+
 #int libusb_init(libusb_context **ctx);
 libusb_init = libusb.libusb_init
 libusb_init.argtypes = [libusb_context_p_p]
@@ -510,6 +541,25 @@ libusb_exit.restype = None
 libusb_set_debug = libusb.libusb_set_debug
 libusb_set_debug.argtypes = [libusb_context_p, c_int]
 libusb_set_debug.restype = None
+#int libusb_has_capability(uint32_t capability);
+try:
+    libusb_has_capability = libusb.libusb_has_capability
+except AttributeError:
+    def libusb_has_capability(_):
+        return 0
+else:
+    libusb_has_capability.argtypes = [c_uint32]
+try:
+    # Note: Should be equivalent to libusb_error.get (except libusb_error.get
+    # one raises on unknown values).
+    #char *libusb_error_name(int errcode);
+    libusb_error_name = libusb.libusb_error_name
+except AttributeError:
+    def libusb_error_name(errcode):
+        return None
+else:
+    libusb_error_name.argtypes = [c_int]
+    libusb_error_name.restype = c_char_p
 try:
     #char *libusb_strerror(enum libusb_error errcode);
     libusb_strerror = libusb.libusb_strerror
@@ -575,6 +625,15 @@ libusb_get_bus_number.restype = c_uint8
 libusb_get_device_address = libusb.libusb_get_device_address
 libusb_get_device_address.argtypes = [libusb_device_p]
 libusb_get_device_address.restype = c_uint8
+try:
+    #int libusb_get_device_speed(libusb_device *dev);
+    libusb_get_device_speed = libusb.libusb_get_device_speed
+except AttributeError:
+    # Place holder
+    def libusb_get_device_speed(_):
+        return LIBUSB_SPEED_UNKNOWN
+else:
+    libusb_get_device_speed.argtypes = [libusb_device_p]
 #int libusb_get_max_packet_size(libusb_device *dev, unsigned char endpoint);
 libusb_get_max_packet_size = libusb.libusb_get_max_packet_size
 libusb_get_max_packet_size.argtypes = [libusb_device_p, c_uchar]
@@ -874,9 +933,28 @@ libusb_wait_for_event.argtypes = [libusb_context_p, timeval_p]
 #int libusb_handle_events_timeout(libusb_context *ctx, struct timeval *tv);
 libusb_handle_events_timeout = libusb.libusb_handle_events_timeout
 libusb_handle_events_timeout.argtypes = [libusb_context_p, timeval_p]
+#int libusb_handle_events_timeout_completed(libusb_context *ctx,
+#   struct timeval *tv, int *completed);
+try:
+    libusb_handle_events_timeout_completed = libusb.\
+        libusb_handle_events_timeout_completed
+except AttributeError:
+    # No safe replacement possible.
+    pass
+else:
+    libusb_handle_events_timeout_completed.argtypes = [libusb_context_p,
+        timeval_p, c_int_p]
 #int libusb_handle_events(libusb_context *ctx);
 libusb_handle_events = libusb.libusb_handle_events
 libusb_handle_events.argtypes = [libusb_context_p]
+#int libusb_handle_events_completed(libusb_context *ctx, int *completed);
+try:
+    libusb_handle_events_completed = libusb.libusb_handle_events_completed
+except AttributeError:
+    # No safe replacement possible.
+    pass
+else:
+    libusb_handle_events_completed.argtypes = [libusb_context_p, c_int_p]
 #int libusb_handle_events_locked(libusb_context *ctx, struct timeval *tv);
 libusb_handle_events_locked = libusb.libusb_handle_events_locked
 libusb_handle_events_locked.argtypes = [libusb_context_p, timeval_p]
