@@ -19,12 +19,38 @@ Features:
   See USBPoller, USBTransfer and USBTransferHelper.
 """
 
+# unused libusb1 functions, and why:
+
+# libusb_get_active_config_descriptor   : superseeded by getConfiguration + 
+
+# libusb_handle_events_timeout_completed: might be added
+# libusb_handle_events_completed        : likewise
+
+# libusb_has_capability                 : no obvious use so far
+# libusb_error_name                     : likewise
+# libusb_get_config_descriptor_by_value : likewise
+# libusb_control_transfer_get_setup     : likewise
+# libusb_set_iso_packet_lengths         : likewise
+# libusb_strerror                       : likewise (and might become
+#                                         unavailable upstream anyway)
+
+# libusb_ref_device                     : no use
+# libusb_get_device                     : likewise
+
+# libusb_open_device_with_vid_pid       : superseeded by python implementation
+#                                         (required for proper refcounting)
+
+# libusb_get_iso_packet_buffer          : superseeded by
+#                                         get_iso_packet_buffer_list
+# libusb_get_iso_packet_buffer_simple   : likewise
+
 import libusb1
 from ctypes import byref, create_string_buffer, c_int, sizeof, POINTER, \
     create_unicode_buffer, c_wchar, cast, c_uint16, c_ubyte, string_at, \
-    addressof, c_void_p, cdll
+    c_void_p, cdll
 from cStringIO import StringIO
 import sys
+import threading
 from ctypes.util import find_library
 
 __all__ = ['LibUSBContext', 'USBDeviceHandle', 'USBDevice',
@@ -67,10 +93,10 @@ DEFAULT_ASYNC_TRANSFER_ERROR_CALLBACK = lambda x: False
 
 def create_binary_buffer(string_or_len):
     # Prevent ctypes from adding a trailing null char.
-    if isinstance(string_or_len, basestring):
-        result = create_string_buffer(string_or_len, len(string_or_len))
-    else:
+    if isinstance(string_or_len, (int, long)):
         result = create_string_buffer(string_or_len)
+    else:
+        result = create_string_buffer(string_or_len, len(string_or_len))
     return result
 
 class USBTransfer(object):
@@ -539,10 +565,108 @@ class USBTransferHelper(object):
         # Deprecated: to drop
         return self.__transfer.isSubmitted()
 
+class USBPollerThread(threading.Thread):
+    """
+    Implements libusb1 documentation about threaded, asynchronous
+    applications.
+    In short, instanciate this class once (...per LibUSBContext instance), call
+    start() on the instance, and do whatever you need.
+    This thread will be used to execute transfer completion callbacks, and you
+    are free to use libusb1's synchronous API in another thread, and can forget
+    about libusb1 file descriptors.
+
+    See http://libusb.sourceforge.net/api-1.0/mtasync.html .
+    """
+    def __init__(self, context, poller, exc_callback=None):
+        """
+        Create a poller thread for given context.
+        Warning: it will not check if another poller instance was already
+        present for that context, and will replace it.
+
+        poller is a polling instance implementing the following methods:
+        - register(fd, event_flags)
+          event_flags have the same meaning as in poll API (POLLIN & POLLOUT)
+        - unregister(fd)
+        - poll(timeout)
+          timeout being a float in seconds, or None if there is no timeout.
+          Its return value must evaluate to true if there are events to handle,
+          false otherwise.
+        poller should not be used outside this class, and should not have
+        any registered file descriptor.
+
+        exc_callback (callable)
+          Called with a libusb_error value as single parameter when event
+          handling fails.
+          If not given, an USBError will be raised, interrupting the thread.
+        """
+        super(USBPollerThread, self).__init__()
+        self.daemon = True
+        self.__context = context
+        self.__poller = poller
+        self.__fd_set = set()
+        context.setPollFDNotifiers(self._registerFD, self._unregisterFD)
+        for fd, events in context.getPollFDList():
+            self._registerFD(fd, events, None)
+        if exc_callback is not None:
+            self.exceptionHandler = exc_callback
+
+    def __del__(self):
+        self.__context.setPollFDNotifiers(None, None)
+
+    @staticmethod
+    def exceptionHandler(exc):
+        raise exc
+
+    def run(self):
+        # We expect quite some spinning in below loop, so move any unneeded
+        # operation out of it.
+        context = self.__context
+        poll = self.__poller.poll
+        try_lock_events = context.tryLockEvents
+        lock_event_waiters = context.lockEventWaiters
+        wait_for_event = context.waitForEvent
+        unlock_event_waiters = context.unlockEventWaiters
+        event_handling_ok = context.eventHandlingOK
+        unlock_events = context.unlockEvents
+        handle_events_locked = context.handleEventsLocked
+        event_handler_active = context.eventHandlerActive
+        getNextTimeout = context.getNextTimeout
+        exceptionHandler = self.exceptionHandler
+        fd_set = self.__fd_set
+        while fd_set:
+            if try_lock_events():
+                lock_event_waiters()
+                while event_handler_active():
+                    wait_for_event()
+                unlock_event_waiters()
+            else:
+                try:
+                    while event_handling_ok():
+                        if poll(getNextTimeout()):
+                            try:
+                                handle_events_locked()
+                            except libusb1.USBError, exc:
+                                exceptionHandler(exc)
+                finally:
+                    unlock_events()
+
+    def _registerFD(self, fd, events, _):
+        self.__poller.register(fd, events)
+        self.__fd_set.add(fd)
+
+    def _unregisterFD(self, fd, _):
+        self.__fd_set.discard(fd)
+        self.__poller.unregister(fd)
+
 class USBPoller(object):
     """
     Class allowing integration of USB event polling in a file-descriptor
     monitoring event loop.
+
+    WARNING: Do not call "poll" from several threads concurently. Do not use
+    synchronous USB transfers in a thread while "poll" is running. Doing so
+    will result in unnecessarily long pauses in some threads. Opening and/or
+    closing devices while polling can cause race conditions to occur.
     """
     def __init__(self, context, poller):
         """
@@ -1131,64 +1255,8 @@ class USBDevice(object):
             self.getProductID(),
         )
 
-    def reprConfigurations(self):
-        """
-        Get a string representation of device's configurations.
-        Note: opens the device temporarily.
-
-        Deprecated. This is useless. It doesn't even showcases API usage, as
-        it accesses privape properties.
-        """
-        out = StringIO()
-        for config in self.__configuration_descriptor_list:
-            print >> out, 'Configuration %i: %s' % (config.bConfigurationValue,
-                self._getASCIIStringDescriptor(config.iConfiguration))
-            print >> out, '  Max Power: %i mA' % (config.MaxPower * 2, )
-            # TODO: bmAttributes dump
-            for interface_num in xrange(config.bNumInterfaces):
-                interface = config.interface[interface_num]
-                print >> out, '  Interface %i' % (interface_num, )
-                for alt_setting_num in xrange(interface.num_altsetting):
-                    altsetting = interface.altsetting[alt_setting_num]
-                    print >> out, '    Alt Setting %i: %s' % (alt_setting_num,
-                        self._getASCIIStringDescriptor(altsetting.iInterface))
-                    print >> out, '      Class: %02x Subclass: %02x' % \
-                        (altsetting.bInterfaceClass,
-                         altsetting.bInterfaceSubClass)
-                    print >> out, '      Protocol: %02x' % \
-                        (altsetting.bInterfaceProtocol, )
-                    for endpoint_num in xrange(altsetting.bNumEndpoints):
-                        endpoint = altsetting.endpoint[endpoint_num]
-                        print >> out, '      Endpoint %i' % (endpoint_num, )
-                        print >> out, '        Address: %02x' % \
-                            (endpoint.bEndpointAddress, )
-                        attribute_list = []
-                        transfer_type = endpoint.bmAttributes & \
-                            libusb1.LIBUSB_TRANSFER_TYPE_MASK
-                        attribute_list.append(libusb1.libusb_transfer_type(
-                            transfer_type
-                        ))
-                        if transfer_type == \
-                            libusb1.LIBUSB_TRANSFER_TYPE_ISOCHRONOUS:
-                            attribute_list.append(libusb1.libusb_iso_sync_type(
-                                (endpoint.bmAttributes & \
-                                 libusb1.LIBUSB_ISO_SYNC_TYPE_MASK) >> 2
-                            ))
-                            attribute_list.append(libusb1.libusb_iso_usage_type(
-                                (endpoint.bmAttributes & \
-                                 libusb1.LIBUSB_ISO_USAGE_TYPE_MASK) >> 4
-                            ))
-                        print >> out, '        Attributes: %s' % \
-                            (', '.join(attribute_list), )
-                        print >> out, '        Max Packet Size: %i' % \
-                            (endpoint.wMaxPacketSize, )
-                        print >> out, '        Interval: %i' % \
-                            (endpoint.bInterval, )
-                        print >> out, '        Refresh: %i' % \
-                            (endpoint.bRefresh, )
-                        print >> out, '        Sync Address: %02x' % \
-                            (endpoint.bSynchAddress, )
-        return out.getvalue()
+    def __getitem__(self, index):
+        return USBConfiguration(self.__configuration_descriptor_list[index])
 
     def iterConfiguations(self):
         for config in self.__configuration_descriptor_list:
@@ -1242,6 +1310,15 @@ class USBDevice(object):
         """
         return self.device_descriptor.bMaxPacketSize0
 
+    def getMaxPacketSize(self, endpoint):
+        """
+        Get device's max packet size for given endpoint.
+        """
+        result = libusb1.libusb_get_max_packet_size(self.device_p, endpoint)
+        if result < 0:
+            raise libusb1.USBError(result)
+        return result
+
     def getMaxISOPacketSize(self, endpoint):
         """
         Get the maximum size for a single isochronous packet for given
@@ -1322,6 +1399,13 @@ class USBDevice(object):
         """
         return self.device_descriptor.bNumConfigurations
 
+    def getDeviceSpeed(self):
+        """
+        Get device's speed (see libusb1.libusb_speed for possible return
+        values).
+        """
+        return libusb1.libusb_get_device_speed(self.device_p)
+
     def open(self):
         """
         Open device.
@@ -1333,6 +1417,9 @@ class USBDevice(object):
             raise libusb1.USBError(result)
         return USBDeviceHandle(self.__context, handle, self)
 
+_zero_tv = libusb1.timeval(0, 0)
+_zero_tv_p = byref(_zero_tv)
+
 class LibUSBContext(object):
     """
     libusb1 USB context.
@@ -1346,6 +1433,17 @@ class LibUSBContext(object):
     __removed_cb = None
     __libusb_set_pollfd_notifiers = libusb1.libusb_set_pollfd_notifiers
 
+    def _validContext(func):
+        # Defined inside LibUSBContext so we can access "self.__*".
+        def wrapper(self, *args, **kw):
+            self.__context_lock.acquire()
+            try:
+                if self.__context_p is not None:
+                    return func(self, *args, **kw)
+            finally:
+                self.__context_lock.release()
+        return wrapper
+
     def __init__(self):
         """
         Create a new USB context.
@@ -1355,21 +1453,28 @@ class LibUSBContext(object):
         if result:
             raise libusb1.USBError(result)
         self.__context_p = context_p
+        # Used to prevent an exit to cause a segfault if a concurent thread
+        # is still in libusb.
+        self.__context_lock = threading.RLock()
 
     def __del__(self):
         self.exit()
 
+    @_validContext
     def exit(self):
         """
         Close (destroy) this USB context.
+
+        When this function has been called, methods on its instance will
+        become mosty no-ops, returning None.
         """
         context_p = self.__context_p
-        if context_p is not None:
-            self.__libusb_exit(context_p)
-            self.__context_p = None
+        self.__libusb_exit(context_p)
+        self.__context_p = None
         self.__added_cb = None
         self.__removed_cb = None
 
+    @_validContext
     def getDeviceList(self):
         """
         Return a list of all USB devices currently plugged in, as USBDevice
@@ -1403,6 +1508,7 @@ class LibUSBContext(object):
             result = None
         return result
 
+    @_validContext
     def getPollFDList(self):
         """
         Return file descriptors to be used to poll USB events.
@@ -1430,6 +1536,7 @@ class LibUSBContext(object):
             _free(pollfd_p_p)
         return result
 
+    @_validContext
     def handleEvents(self):
         """
         Handle any pending event (blocking).
@@ -1440,6 +1547,9 @@ class LibUSBContext(object):
         if result:
             raise libusb1.USBError(result)
 
+    # TODO: handleEventsCompleted
+
+    @_validContext
     def handleEventsTimeout(self, tv=0):
         """
         Handle any pending event.
@@ -1457,6 +1567,9 @@ class LibUSBContext(object):
         if result:
             raise libusb1.USBError(result)
 
+    # TODO: handleEventsTimeoutCompleted
+
+    @_validContext
     def setPollFDNotifiers(self, added_cb=None, removed_cb=None,
             user_data=None):
         """
@@ -1478,6 +1591,7 @@ class LibUSBContext(object):
         self.__libusb_set_pollfd_notifiers(self.__context_p, added_cb,
                                             removed_cb, user_data)
 
+    @_validContext
     def getNextTimeout(self):
         """
         Returns the next internal timeout that libusb needs to handle, in
@@ -1496,6 +1610,7 @@ class LibUSBContext(object):
             raise libusb1.USBError(result)
         return result
 
+    @_validContext
     def setDebug(self, level):
         """
         Set debugging level.
@@ -1504,3 +1619,71 @@ class LibUSBContext(object):
         """
         libusb1.libusb_set_debug(self.__context_p, level)
 
+    @_validContext
+    def tryLockEvents(self):
+        """
+        See libusb_try_lock_events doc.
+        """
+        return libusb1.libusb_try_lock_events(self.__context_p)
+
+    @_validContext
+    def lockEvents(self):
+        """
+        See libusb_lock_events doc.
+        """
+        libusb1.libusb_lock_events(self.__context_p)
+
+    @_validContext
+    def lockEventWaiters(self):
+        """
+        See libusb_lock_event_waiters doc.
+        """
+        libusb1.libusb_lock_event_waiters(self.__context_p)
+
+    @_validContext
+    def waitForEvent(self):
+        """
+        See libusb_wait_for_event doc.
+        """
+        libusb1.libusb_wait_for_event(self.__context_p)
+
+    @_validContext
+    def unlockEventWaiters(self):
+        """
+        See libusb_unlock_event_waiters doc.
+        """
+        libusb1.libusb_unlock_event_waiters(self.__context_p)
+
+    @_validContext
+    def eventHandlingOK(self):
+        """
+        See libusb_event_handling_ok doc.
+        """
+        return libusb1.libusb_event_handling_ok(self.__context_p)
+
+    @_validContext
+    def unlockEvents(self):
+        """
+        See libusb_unlock_events doc.
+        """
+        libusb1.libusb_unlock_events(self.__context_p)
+
+    @_validContext
+    def handleEventsLocked(self):
+        """
+        See libusb_handle_events_locked doc.
+        """
+        # XXX: does tv parameter need to be exposed ?
+        result = libusb1.libusb_handle_events_locked(self.__context_p, _zero_tv_p)
+        if result:
+            raise libusb1.USBError(result)
+
+    @_validContext
+    def eventHandlerActive(self):
+        """
+        See libusb_event_handler_active doc.
+        """
+        return libusb1.libusb_event_handler_active(self.__context_p)
+
+del LibUSBContext._validContext
+
